@@ -73,8 +73,7 @@ def read_abs(path: Union[str, Path]
     p = Path(path)
     # Figure out which type of spectrum it is.
     if not (p / "procs").exists() or not (p.parents[1] / "acqus").exists():
-        raise FileNotFoundError(f"Invalid path to spectrum {p}:"
-                                " either procs or acqus were not found")
+        raise FileNotFoundError(f"Invalid path to spectrum {p}")
     if (p.parents[1] / "ser").exists():
         if (p / "used_from").exists():
             return Dataset1DProj(p)
@@ -172,7 +171,7 @@ RSEL SI STSI STSR SYMM TD TD0 TDEFF TDOFF TILT WBST WDW XDIM XGAIN YMAX_P
 YMIN_P""".split()
 
 # Extra integer parameters that weren't documented.
-_int_pars = _int_pars + """DTYPA DTYPP GRPDLY""".split()
+_int_pars = _int_pars + """DTYPA DTYPP""".split()
 
 
 # -- Fundamental Dataset methods ------------------------
@@ -427,6 +426,10 @@ class _1D_RawDataMixin():
             self._read_raw_data()
             return self._fid
 
+    @fid.setter
+    def fid(self, value):
+        self._fid = value
+
     def _find_raw_data_paths(self) -> None:
         self.path: Path
         self._p_fid = self.path.parents[1] / "fid"
@@ -472,12 +475,12 @@ class _1D_ProcDataMixin():
         chemical shift.
 
     imag : ndarray
-        This attribute only applies to `Dataset1D` instances. The projection
-        classes do not have this attribute.
-
         Real-valued |ndarray| containing the imaginary spectrum. ``imag[0]``
         contains the left-most point of the spectrum, i.e. the greatest
         chemical shift.
+
+        This attribute only applies to `Dataset1D` instances. The projection
+        classes do not have this attribute.
     """
 
     @property
@@ -488,6 +491,10 @@ class _1D_ProcDataMixin():
             self._read_spec(spectype="real")
             return self._real
 
+    @real.setter
+    def real(self, value):
+        self._real = value
+
     @property
     def imag(self):
         try:
@@ -496,6 +503,10 @@ class _1D_ProcDataMixin():
             self._read_spec(spectype="imag")
             return self._imag
 
+    @imag.setter
+    def imag(self, value):
+        self._imag = value
+
     def _find_proc_data_paths(self) -> None:
         self._p_real = self.path / "1r"        # type: ignore # mixin
         self._p_imag = self.path / "1i"        # type: ignore # mixin
@@ -663,6 +674,10 @@ class _2D_RawDataMixin():
             self._read_raw_data()
             return self._ser
 
+    @ser.setter
+    def ser(self, value):
+        self._ser = value
+
     def _find_raw_data_paths(self) -> None:
         self.path: Path
         self._p_ser = self.path.parents[1] / "ser"
@@ -712,6 +727,10 @@ class _2D_ProcDataMixin():
             self._read_spec(spectype="rr")
             return self._rr
 
+    @rr.setter
+    def rr(self, value):
+        self._rr = value
+
     @property
     def ri(self):
         try:
@@ -720,6 +739,10 @@ class _2D_ProcDataMixin():
             self._read_spec(spectype="ri")
             return self._ri
 
+    @ri.setter
+    def ri(self, value):
+        self._ri = value
+
     @property
     def ir(self):
         try:
@@ -728,6 +751,10 @@ class _2D_ProcDataMixin():
             self._read_spec(spectype="ir")
             return self._ir
 
+    @ir.setter
+    def ir(self, value):
+        self._ir = value
+
     @property
     def ii(self):
         try:
@@ -736,6 +763,10 @@ class _2D_ProcDataMixin():
             self._read_spec(spectype="ii")
             return self._ii
 
+    @ii.setter
+    def ii(self, value):
+        self._ii = value
+
     @property
     def ts_baselev(self):
         """TopSpin base level for contours."""
@@ -1415,21 +1446,30 @@ class Dataset2D(_2D_RawDataMixin,
         return self.sum(axis="row", bounds=bounds)
 
     def slice(self,
-              axis: Union[int, str],
-              ppm: float,
+              axis: Optional[Union[int, str]] = None,
+              ppm: OF = None,
+              f1: OF = None,
+              f2: OF = None,
               ) -> Dataset1DProjVirtual:
-        """Extract a 1D slice from a 2D spectrum.
+        """Extract a 1D slice from a 2D spectrum. You must either specify both
+        *axis* and *ppm* arguments, or *f1* only, or *f2* only.
 
         Parameters
         ----------
-        axis : int or str from {0, "column", 1, "row"}
-            Axis to slice along. To extract a column (which corresponds to a
-            slice along f1, at one specific value of f2), use ``0`` or
-            ``column``, and vice versa for ``row``.
-        ppm : float
-            The chemical shift of the other axis to slice at. For example, if
-            you are extracting a column, then this would be the f2 chemical
-            shift of interest.
+        axis : str from {"column", "row"}, optional
+            Axis to slice along. To extract a column (i.e. at one particular
+            value of f2), use ``column``, and vice versa.
+        ppm : float, optional
+            The chemical shift to slice at. For example, if you are extracting
+            a column, then this would be the f2 chemical shift of interest.
+        f1 : float, optional
+            slice(f1=y) is an alias for slice(axis="row", ppm=y). If
+            specified, this overrules the *axis* and *ppm* keyword arguments.
+            Cannot be used together with *f2*.
+        f2 : float, optional
+            slice(f2=x) is an alias for slice(axis="column", ppm=x). If specified,
+            this overrules the *axis* and *ppm* keyword arguments. Cannot be
+            used together with *f1*.
 
         Returns
         -------
@@ -1439,17 +1479,29 @@ class Dataset2D(_2D_RawDataMixin,
             same manner. The actual projection or sum can be accessed using
             `_1D_ProcDataMixin.proc_data`, which `Dataset1DProj` inherits.
         """
-        # Find axis
-        if axis == "column":  # extract a column
-            axis = 0
-        elif axis == "row":   # extract a row
-            axis = 1
-        if axis not in [0, 1]:
-            raise ValueError(f"Invalid value for axis '{axis}'")
-        # For some reason mypy doesn't realise that axis must be an int by here.
-        index = self.ppm_to_index(axis=(1 - axis), ppm=ppm)   # type: ignore
+        # Check f1 and f2 keyword arguments.
+        if f1 is not None and f2 is not None:
+            raise ValueError("'f1' and 'f2' keyword arguments cannot be used"
+                             " simultaneously.")
+        elif f1 is not None:
+            axis_index, ppm = 1, f1
+        elif f2 is not None:
+            axis_index, ppm = 0, f2
+        # If we reach this else block, then neither of them were passed.
+        else:
+            if axis is None or ppm is None:
+                raise ValueError("slice() requires either the 'axis' and 'ppm'"
+                                 " keyword arguments, or one of 'f1' or 'f2'.")
+            if axis not in ["column", "row"]:
+                raise ValueError(f"Invalid value for axis '{axis}'")
+            # Determine axis_index from axis argument.
+            axis_index = 1 if axis == "row" else 0
+
+        # Generate the virtual projection, then return it
+        shift_index = self.ppm_to_index(axis=(1 - axis_index), ppm=ppm)   # type: ignore
         return Dataset1DProjVirtual(self.path, proj_type="slice",
-                                    proj_axis=axis, index=index,
+                                    proj_axis=axis_index,
+                                    index=shift_index,
                                     rr=self.rr)
 
 
