@@ -437,6 +437,10 @@ def _stage1d(dataset: ds.TDataset1D,
                                 color=color,
                                 plot_options=kwargs)
         ax.pha.plot_objs.append(plot_obj)
+    # Make the Axes the currently active one so that mkplot() will
+    # automatically construct the correct Axes, even if the ax parameter is not
+    # passed to it.
+    plt.sca(ax)
 
 
 class PlotObject1D():
@@ -884,6 +888,10 @@ def _stage2d(dataset: ds.Dataset2D,
                                 dfilter=dfilter, label=label,
                                 plot_options=kwargs)
         ax.pha.plot_objs.append(plot_obj)
+    # Make the Axes the currently active one so that mkplot() will
+    # automatically construct the correct Axes, even if the ax parameter is not
+    # passed to it.
+    plt.sca(ax)
 
 
 def _mkplot2d(ax: Any = None,
@@ -1237,15 +1245,27 @@ def subplots2d(nrows: int = 1,
                              " elements as there are rows.")
         # Calculate the width/height.
         if width_ratios is not None:
-            fig_width = np.sum(width_ratios) * 4
+            if "figsize" in kwargs:
+                fig_width = kwargs["figsize"][0]
+            else:
+                fig_width = np.sum(width_ratios) * 4
             gridspec_kw["width_ratios"] = width_ratios
         else:
-            fig_width = ncols * 4
+            if "figsize" in kwargs:
+                fig_width = kwargs["figsize"][0]
+            else:
+                fig_width = ncols * 4
         if height_ratios is not None:
-            fig_height = np.sum(height_ratios) * 4
+            if "figsize" in kwargs:
+                fig_height = kwargs["figsize"][1]
+            else:
+                fig_height = np.sum(height_ratios) * 4
             gridspec_kw["height_ratios"] = height_ratios
         else:
-            fig_height = nrows * 4
+            if "figsize" in kwargs:
+                fig_height = kwargs["figsize"][1]
+            else:
+                fig_height = nrows * 4
         figsize = (fig_width, fig_height)
         # Remove figsize from kwargs if it's there.
         kwargs.pop("figsize", None)
