@@ -178,6 +178,93 @@ class Hsqc(Experiment):
         return df
 
 
+@export
+class HsqcCosy(Experiment):
+    """
+    For 13C HSQC-COSY experiments. The variables hsqc and cosy should be lists
+    of 2-tuples (f1_shift, f2_shift) which indicate the direct (HSQC) and
+    indirect (HSQC-COSY) responses respectively.
+
+    None of the methods from Experiment are actually inherited.
+    """
+    def __init__(self,
+                 hsqc: List[Tuple[float, float]],
+                 cosy: List[Tuple[float, float]],
+                 margin: Optional[Tuple[float, float]] = (0.5, 0.02),
+                 ):
+        self.hsqc = hsqc
+        self.cosy = cosy
+        self.margin = margin
+
+    @property
+    def peaks(self) -> List[Tuple[float, float]]:
+        """
+        Returns a list of all peaks.
+        """
+        return self.hsqc + self.cosy
+
+    @property
+    def df(self) -> pd.DataFrame:
+        """
+        Return a pandas DataFrame containing all the peaks. This DF has
+        columns "f1", "f2", and "type".
+        """
+        hsqc_df, cosy_df = (
+            pd.DataFrame.from_records(peaklist, columns=("f1", "f2"))
+            for peaklist in (self.hsqc, self.cosy)
+        )
+        hsqc_df["type"] = "hsqc"
+        cosy_df["type"] = "cosy"
+        return pd.concat((hsqc_df, cosy_df), ignore_index=True)
+
+    def integrate(self,
+                  dataset: ds.Dataset2D,
+                  edited: bool = True,
+                  ) -> np.ndarray:
+        """
+        Calculates the absolute integral of each peak in the HSQC. If editing
+        is enabled, assumes that HSQC peaks are positive and HSQC-COSY peaks
+        negative.
+        """
+        if edited:
+            # We need self.df here as it contains multiplicity information.
+            return np.array([dataset.integrate(peak=(peak.f1, peak.f2),
+                                               margin=self.margin,
+                                               mode=("max"
+                                                     if peak.type == "hsqc"
+                                                     else "min"))
+                             for peak in self.df.itertuples()])
+        else:
+            return np.array([dataset.integrate(peak=peak,
+                                               margin=self.margin,
+                                               mode=("max"))
+                             for peak in self.peaks])
+
+    def rel_ints_df(self,
+                    dataset: ds.Dataset2D,
+                    ref_dataset: ds.Dataset2D,
+                    label: str = "",
+                    edited: bool = True,
+                    ) -> pd.DataFrame:
+        """
+        Construct a dataframe of relative intensities vs a reference
+        dataset.
+
+        This DataFrame will have columns (f1, f2, mult) just like self.df,
+        but will also have "expt" which is a string indicating the type of
+        experiment being ran, and "int" which is the relative integral vs a
+        reference dataset.
+        """
+        df = pd.DataFrame()
+        df["int"] = (self.integrate(dataset, edited=edited) /
+                     self.integrate(ref_dataset, edited=edited))
+        df["expt"] = label
+        df["type"] = self.df["type"]
+        df["f1"] = self.df["f1"]
+        df["f2"] = self.df["f2"]
+        return df
+
+
 @export
 class Cosy(Experiment):
     """
@@ -300,28 +387,41 @@ class Andrographolide():
                        (1.2131, 1.6434), (1.2131, 1.3500), (1.3696, 2.3181)]
     cosy = Cosy(cosy_diagonal, cosy_cross_half)
 
+    # HSQC-COSY spectrum, we treat it as a HSQC-COSY for now
+    hsqc_cosy_hsqc = hsqc_ch + hsqc_ch2 + hsqc_ch3
+    hsqc_cosy_cosy = [(24.3311, 1.2059), (28.3694, 1.2156), (24.5067, 1.9386),
+                      (24.5067, 1.8702), (28.3694, 1.7090), (36.9727, 1.6455),
+                      (38.0262, 1.3524), (54.8816, 1.3671), (55.9351, 2.5101),
+                      (28.3694, 3.2331), (24.5067, 2.3147), (37.8506, 1.7383),
+                      (24.5067, 6.6280), (64.8896, 5.7146), (74.7219, 4.9183),
+                      (65.0651, 4.4006), (78.9358, 1.6455), (146.7087, 2.4889)]
+    hsqc_cosy = HsqcCosy(hsqc_cosy_hsqc, hsqc_cosy_cosy)
+
 
 @export
 class Zolmitriptan():
     """
     50 mM in DMSO. HMBC, 15N HSQC, 13C HSQC data available.
     """
+    # I redid the peak picking on 21/5/21 because the ones above didn't seem
+    # accurate enough.
     hmbc_peaks = [
-        (159.1747, 7.7741), (159.1747, 4.2335), (159.1747, 4.0397),
-        (135.6473, 10.7041), (135.6473, 7.3631), (135.6473, 7.1165),
-        (135.6473, 6.9345), (127.9219, 10.7041), (127.9219, 7.2574),
-        (127.9219, 7.1165), (127.9219, 2.8067), (126.3417, 7.2574),
-        (126.1661, 2.9006), (126.1661, 2.8184), (126.1661, 2.7656),
-        (123.1813, 10.6982), (123.1813, 2.8067), (123.0057, 7.3631),
-        (123.0057, 2.9006), (119.1430, 6.9345), (119.1430, 2.9006),
-        (119.1430, 2.8184), (119.1430, 2.7656), (112.8222, 10.7041),
-        (112.8222, 7.3631), (112.8222, 7.1165), (112.8222, 2.8008),
-        (112.8222, 2.5307), (68.5766, 7.7741), (68.5766, 2.9006),
-        (68.5766, 2.8125), (68.5766, 2.7656), (60.3245, 2.8008),
-        (60.3245, 2.2312), (53.6525, 7.7741), (53.6525, 2.8947),
-        (53.6525, 2.8125), (53.6525, 2.7597), (45.5760, 2.5307),
-        (45.5760, 2.2312), (41.0110, 7.3631), (41.0110, 6.9345),
-        (41.0110, 4.2335), (41.0110, 4.0397)
+        (135.6473, 10.7204), (123.1813, 10.7204), (112.4710, 10.7204),
+        (127.9219, 10.7204), (41.0110, 6.9396), (53.6525, 7.7798),
+        (41.0110, 7.3694), (68.5766, 7.7798), (112.4710, 7.3646),
+        (127.9219, 7.2620), (112.4710, 7.1203), (127.9219, 7.1252),
+        (135.6473, 7.1252), (126.3417, 7.2620), (119.1430, 6.9347),
+        (123.0057, 7.3694), (135.6473, 7.3694), (135.6473, 6.9396),
+        (159.1747, 7.7798), (41.0110, 4.0380), (41.0110, 4.2334),
+        (53.6525, 4.0282), (68.5766, 2.8070), (68.5766, 2.7679),
+        (53.6525, 2.8998), (53.6525, 2.7679), (53.6525, 2.8070),
+        (68.5766, 2.8998), (60.1489, 2.8217), (45.4004, 2.5872),
+        (60.1489, 2.2746), (45.4004, 2.2746), (159.1747, 4.2334),
+        (159.1747, 4.0380), (123.0057, 2.8998), (119.1430, 2.8998),
+        (119.3186, 2.8070), (123.0057, 2.7679), (112.4710, 2.5872),
+        (126.3417, 2.8119), (126.3417, 2.8998), (119.1430, 2.7679),
+        (126.3417, 2.7728), (127.9219, 2.8217), (123.3569, 2.8217),
+        (112.4710, 2.8217), (23.2776, 2.5872)
     ]
     hmbc = Hmbc(hmbc_peaks)
 
@@ -540,6 +640,7 @@ def cosy_stripplot(molecule: Any,
                    ylabel: str = "Intensity",
                    title: str = "",
                    ncol: int = 2,
+                   separate_type: bool = True,
                    loc: str = "upper center",
                    ax: Optional[Any] = None,
                    **kwargs: Any,
@@ -586,6 +687,9 @@ def cosy_stripplot(molecule: Any,
                                               ref_dataset=ref_dataset,
                                               label=label)
                     for (ds, label) in zip(datasets, expt_labels)]
+    if not separate_type:
+        rel_ints_dfs = [rel_int_df.assign(type="cosy")
+                        for rel_int_df in rel_ints_dfs]
     all_dfs = pd.concat(rel_ints_dfs)
 
     # Calculate the average integrals by type
@@ -603,8 +707,11 @@ def cosy_stripplot(molecule: Any,
                   palette=sns.color_palette("deep")[3:], **kwargs)
     # Customise the plot
     ax.set(xlabel=xlabel, ylabel=ylabel, title=title)
-    ax.legend(ncol=ncol, loc=loc,
-              labels=["diagonal", "cross"]).set(title=None)
+    if separate_type:
+        ax.legend(ncol=ncol, loc=loc,
+                  labels=["diagonal", "cross"]).set(title=None)
+    else:
+        ax.legend().set_visible(False)
     ax.axhline(y=1, color="grey", linewidth=0.5, linestyle="--")
     # Set y-limits. We need to expand it by ~20% to make space for the legend,
     # as well as the averaged values.
@@ -616,10 +723,12 @@ def cosy_stripplot(molecule: Any,
     new_ymax = ymean + (EXPANSION_FACTOR * ylength)
     ax.set_ylim((new_ymin, new_ymax))
     # add the text
+    offset = -0.2 if separate_type else 0
+    dx = 0.4 if separate_type else 1
     for x, (_, expt_avgs) in enumerate(avgd_ints.items()):
         for i, ((_, avg), color) in enumerate(zip(
                 expt_avgs.items(), sns.color_palette("deep")[3:])):
-            ax.text(x=x-0.2+i*0.4, y=0.02, s=f"({avg:.2f})",
+            ax.text(x=x-offset+i*dx, y=0.02, s=f"({avg:.2f})",
                     color=color, horizontalalignment="center",
                     transform=ax.get_xaxis_transform())
     style_axes(ax, "plot")
@@ -748,3 +857,114 @@ def hsqc_cosy_stripplot(molecule: Any,
                     **font_kwargs)
     style_axes(ax, "plot")
     return plt.gcf(), ax
+
+
+@export
+def hsqcc_stripplot(molecule: Any,
+                    datasets: Union[ds.Dataset2D, Sequence[ds.Dataset2D]],
+                    ref_dataset: ds.Dataset2D,
+                    expt_labels: Union[str, Sequence[str]],
+                    xlabel: str = "Experiment",
+                    ylabel: str = "Intensity",
+                    title: str = "",
+                    edited: bool = True,
+                    show_averages: bool = True,
+                    ncol: int = 2,
+                    loc: str = "upper center",
+                    ax: Optional[Any] = None,
+                    **kwargs: Any,
+                    ) -> Tuple[Any, Any]:
+    """
+    Plot HSQC-COSY strip plots (i.e. plot relative intensities, split by peak
+    type).
+
+    Parameters
+    ----------
+    molecule : pg.private.Andrographolide
+        The class from which the hsqc attribute will be taken from
+    datasets : pg.Dataset2D or sequence of pg.Dataset2D
+        Dataset(s) to analyse intensities of
+    ref_dataset : pg.Dataset2D
+        Reference dataset
+    expt_labels : str or sequence of strings
+        Labels for the analysed datasets
+    xlabel : str, optional
+        Axes x-label, defaults to "Experiment"
+    ylabel : str, optional
+        Axes y-label, defaults to "Intensity"
+    title : str, optional
+        Axes title, defaults to empty string
+    edited : bool, default False
+        Whether editing is enabled or not.
+    show_averages : bool, default True
+        Whether to indicate averages in each category using sns.pointplot.
+    ncol : int, optional
+        Passed to ax.legend(). Defaults to 2.
+    loc : str, optional
+        Passed to ax.legend(). Defaults to "upper center".
+    ax : matplotlib.axes.Axes, optional
+        Axes instance to plot on. If not provided, uses plt.gca().
+    kwargs : dict, optional
+        Keywords passed on to sns.stripplot().
+
+    Returns
+    -------
+    (fig, ax).
+    """
+    # Stick dataset/label into a list if needed
+    if isinstance(datasets, ds.Dataset2D):
+        datasets = [datasets]
+    if isinstance(expt_labels, str):
+        expt_labels = [expt_labels]
+    # Calculate dataframes of relative intensities.
+    rel_ints_dfs = [molecule.hsqc_cosy.rel_ints_df(dataset=ds,
+                                                   ref_dataset=ref_dataset,
+                                                   label=label,
+                                                   edited=edited)
+                    for (ds, label) in zip(datasets, expt_labels)]
+    all_dfs = pd.concat(rel_ints_dfs)
+
+    # Calculate the average integrals by multiplicity
+    avgd_ints = pd.concat((df.groupby("type").mean() for df in rel_ints_dfs),
+                          axis=1)
+    avgd_ints.drop(columns=["f1", "f2"], inplace=True)
+
+    # Get currently active axis if none provided
+    if ax is None:
+        ax = plt.gca()
+
+    # Plot the intensities.
+    stripplot_alpha = 0.3 if show_averages else 0.8
+    sns.stripplot(x="expt", y="int", hue="type",
+                  zorder=0, alpha=stripplot_alpha,
+                  dodge=True, data=all_dfs, ax=ax, **kwargs)
+    if show_averages:
+        sns.pointplot(x="expt", y="int", hue="type", zorder=1,
+                      dodge=0.4, data=all_dfs, ax=ax, join=False,
+                      markers='_', palette="dark", ci=None, scale=1.25)
+    # Customise the plot
+    ax.set(xlabel=xlabel, ylabel=ylabel, title=title)
+    handles, _ = ax.get_legend_handles_labels()
+    l = ax.legend(ncol=ncol, loc=loc,
+                  markerscale=0.4,
+                  handles=handles[0:3],
+                  labels=["direct", "indirect"])
+    ax.axhline(y=1, color="grey", linewidth=0.5, linestyle="--")
+    # Set y-limits. We need to expand it by ~20% to make space for the legend,
+    # as well as the averaged values.
+    EXPANSION_FACTOR = 1.2
+    ymin, ymax = ax.get_ylim()
+    ymean = (ymin + ymax)/2
+    ylength = (ymax - ymin)/2
+    new_ymin = ymean - (EXPANSION_FACTOR * ylength)
+    new_ymax = ymean + (EXPANSION_FACTOR * ylength)
+    ax.set_ylim((new_ymin, new_ymax))
+    # add the text
+    for x, (_, expt_avgs) in enumerate(avgd_ints.items()):
+        for i, ((_, avg), color) in enumerate(zip(expt_avgs.items(),
+                                                  sns.color_palette("deep"))):
+            ax.text(x=x-0.2+i*0.4, y=0.02, s=f"({avg:.2f})",
+                    color=color, horizontalalignment="center",
+                    transform=ax.get_xaxis_transform())
+    style_axes(ax, "plot")
+    return plt.gcf(), ax
